<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 502 $
 * $Id: ClientContactFactory.class.php 502 2007-01-06 00:52:50Z ipso $
 * $Date: 2007-01-05 16:52:50 -0800 (Fri, 05 Jan 2007) $
 */
class ClientContactFactory extends Factory {
	protected $table = 'client_contact';
	protected $pk_sequence_name = 'client_contact_id_seq'; //PK Sequence name

	protected $client_obj = NULL;

	protected $type_options = array(
										10 => 'Billing',
										20 => 'Shipping',
										25 => 'Technical',
										30 => 'Other'
									);

	protected $client_validator_msg = 'Client is invalid';
	protected $client_validator_label = 'client';

	protected $type_validator_options_msg = 'Incorrect Type';
	protected $type_validator_label = 'type';

	protected $name_validator_regex = '/^[a-zA-Z -\.]{1,250}$/i';

	protected $firstname_validator_regex_msg = 'First name contains invalid characters';
	protected $firstname_validator_length_msg = 'First name is too short or too long';
	protected $firstname_validator_label = 'first_name';

	protected $lastname_validator_regex_msg = 'Last name contains invalid characters';
	protected $lastname_validator_length_msg = 'Last name is too short or too long';
	protected $lastname_validator_label = 'last_name';

	protected $companyname_validator_length_msg = 'Company Name is too short or too long';
	protected $companyname_validator_label = 'company_name';

	protected $address_validator_regex = '/^[a-zA-Z0-9-_\.#\ ]{1,250}$/i';

	protected $address1_validator_regex_msg = 'Address1 contains invalid characters';
	protected $address1_validator_length_msg = 'Address1 is too short or too long';
	protected $address1_validator_label = 'address1';

	protected $address2_validator_regex_msg = 'Address2 contains invalid characters';
	protected $address2_validator_length_msg = 'Address2 is too short or too long';
	protected $address2_validator_label = 'address2';

	protected $city_validator_regex = '/^[a-zA-Z0-9-\.\ ]{1,250}$/i';
	protected $city_validator_regex_msg = 'City contains invalid characters';
	protected $city_validator_length_msg = 'City name is too short or too long';
	protected $city_validator_label = 'city';

	protected $province_validator_options_msg = 'Invalid Province';
	protected $province_validator_label = 'province';

	protected $country_validator_options_msg = 'Invalid Country';
	protected $country_validator_label = 'country';

	protected $invoicedistrict_validator_msg = 'Invoice District is invalid';
	protected $invoicedistrict_validator_label = 'invoice_district';

	protected $postalcode_validator_regex_msg = 'Postal Code contains invalid characters, or invalid format';
	protected $postalcode_validator_length_msg = 'Postal Code is too short or too long';
	protected $postalcode_validator_label = 'postal_code';

	protected $workphone_validator_isphonenumber_msg = 'Work phone number is invalid';
	protected $workphone_validator_label = 'work_phone';

	protected $workphoneext_validator_length_msg = 'Work phone number extension is too short or too long';
	protected $workphoneext_validator_label = 'work_phone_ext';

	protected $mobilephone_validator_isphonenumber_msg = 'Mobile phone number is invalid';
	protected $mobilephone_validator_label = 'mobile_phone';

	protected $faxphone_validator_isphonenumber_msg = 'Fax phone number is invalid';
	protected $faxphone_validator_label = 'fax_phone';

	protected $email_validator_isemail_msg = 'Email address is invalid';
	protected $email_validator_label = 'email';

	protected $default_validator_unique_msg = 'There is already a default contact for this type';
	protected $default_validator_label = 'default';

	protected $otherid1_validator_islength_msg = 'Other ID 1 is invalid';
	protected $otherid1_validator_label = 'other_id1';

	protected $otherid2_validator_islength_msg = 'Other ID 2 is invalid';
	protected $otherid2_validator_label = 'other_id2';

	protected $otherid3_validator_islength_msg = 'Other ID 3 is invalid';
	protected $otherid3_validator_label = 'other_id3';

	protected $otherid4_validator_islength_msg = 'Other ID 4 is invalid';
	protected $otherid4_validator_label = 'other_id4';

	protected $otherid5_validator_islength_msg = 'Other ID 5 is invalid';
	protected $otherid5_validator_label = 'other_id5';

	protected $note_validator_length_msg = 'Note is too short or too long';
	protected $note_validator_label = 'note';

	protected $includetaxpolicy_validator_msg = 'Selected Include Tax Policy is invalid';
	protected $includetaxpolicy_validator_label = 'include_tax_policy';

	protected $excludetaxpolicy_validator_msg = 'Selected Exclude Tax Policy is invalid';
	protected $excludetaxpolicy_validator_label = 'exclude_tax_policy';

	function getClientObject() {
		if ( is_object($this->client_obj) ) {
			return $this->client_obj;
		} else {
			$clf = new ClientListFactory();
			$clf->getById( $this->getClient() );
			if ( $clf->getRecordCount() > 0 ) {
				$this->client_obj = $clf->getCurrent();
				return $this->client_obj;
			}

			return FALSE;
		}
	}

	function getClient() {
		if ( isset($this->data['client_id']) ) {
			return $this->data['client_id'];
		}

		return FALSE;
	}
	function setClient($id) {
		$id = trim($id);

		Debug::Text('Client ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$clf = new ClientListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->client_validator_label,
													$clf->getByID($id),
													$this->client_validator_msg
													) ) {

			$this->data['client_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getType() {
		if ( isset($this->data['type_id']) ) {
			return $this->data['type_id'];
		}

		return FALSE;
	}
	function setType($type) {
		$type = trim($type);

		$key = Option::getByValue($type, $this->getOptions('type') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->type_validator_label,
											$type,
											$this->type_validator_options_msg,
											$this->getOptions('type')) ) {

			$this->data['type_id'] = $type;

			return FALSE;
		}

		return FALSE;
	}

	function getFullName($reverse = FALSE) {
		if ( isset($this->data['first_name']) ) {
			if ( $reverse === TRUE ) {
				return $this->data['last_name'].', '.$this->data['first_name'];
			} else {
				return $this->data['first_name'].' '.$this->data['last_name'];
			}
		}

		return FALSE;
	}

	function getFirstName() {
		if ( isset($this->data['first_name']) ) {
			return $this->data['first_name'];
		}

		return FALSE;
	}
	function setFirstName($first_name) {
		$first_name = trim($first_name);

		if 	(	$this->Validator->isRegEx(		$this->firstname_validator_label,
												$first_name,
												$this->firstname_validator_regex_msg,
												$this->name_validator_regex)
				AND
					$this->Validator->isLength(		$this->firstname_validator_label,
													$first_name,
													$this->firstname_validator_length_msg,
													2,
													50) ) {

			$this->data['first_name'] = $first_name;

			return TRUE;
		}

		return FALSE;
	}

	function getLastName() {
		if ( isset($this->data['last_name']) ) {
			return $this->data['last_name'];
		}

		return FALSE;
	}
	function setLastName($last_name) {
		$last_name = trim($last_name);

		if 	(	$this->Validator->isRegEx(		$this->lastname_validator_label,
												$last_name,
												$this->lastname_validator_regex_msg,
												$this->name_validator_regex)
				AND
					$this->Validator->isLength(		$this->lastname_validator_label,
													$last_name,
													$this->lastname_validator_length_msg,
													2,
													50) ) {

			$this->data['last_name'] = $last_name;

			return TRUE;
		}

		return FALSE;
	}

	function getCompanyName() {
		return $this->data['company_name'];
	}
	function setCompanyName($name) {
		$name = trim($name);

		if 	(	$this->Validator->isLength(		$this->name_validator_label,
												$name,
												$this->name_validator_length_msg,
												2,
												100) ) {

			$this->data['company_name'] = $name;

			return TRUE;
		}

		return FALSE;
	}

	function getAddress1() {
		if ( isset($this->data['address1']) ) {
			return $this->data['address1'];
		}

		return FALSE;
	}
	function setAddress1($address1) {
		$address1 = trim($address1);

		if 	(	$address1 != ''
				AND ( $this->Validator->isRegEx(		$this->address1_validator_label,
												$address1,
												$this->address1_validator_regex_msg,
												$this->address_validator_regex)
				AND
					$this->Validator->isLength(		$this->address1_validator_label,
													$address1,
													$this->address1_validator_length_msg,
													2,
													250) ) ) {

			$this->data['address1'] = $address1;

			return TRUE;
		}

		return FALSE;
	}

	function getAddress2() {
		if ( isset($this->data['address2']) ) {
			return $this->data['address2'];
		}

		return FALSE;
	}
	function setAddress2($address2) {
		$address2 = trim($address2);

		if 	(	$address2 != ''
				AND (
					$this->Validator->isRegEx(		$this->address2_validator_label,
													$address2,
													$this->address2_validator_regex_msg,
													$this->address_validator_regex)
				AND
					$this->Validator->isLength(		$this->address2_validator_label,
													$address2,
													$this->address2_validator_length_msg,
													2,
													250) ) ) {

			$this->data['address2'] = $address2;

			return TRUE;
		}

		return FALSE;

	}

	function getCity() {
		if ( isset($this->data['city']) ) {
			return $this->data['city'];
		}

		return FALSE;
	}
	function setCity($city) {
		$city = trim($city);

		if 	(	$this->Validator->isRegEx(		$this->city_validator_label,
												$city,
												$this->city_validator_regex_msg,
												$this->city_validator_regex)
				AND
					$this->Validator->isLength(		$this->city_validator_label,
													$city,
													$this->city_validator_length_msg,
													2,
													250) ) {

			$this->data['city'] = $city;

			return TRUE;
		}

		return FALSE;
	}

	function getProvince() {
		if ( isset($this->data['province']) ) {
			return $this->data['province'];
		}

		return FALSE;
	}
	function setProvince($province) {
		$province = trim($province);

		Debug::Text('Country: '. $this->getCountry() .' Province: '. $province, __FILE__, __LINE__, __METHOD__,10);

		$cf = new CompanyFactory();

		$options_arr = $cf->getOptions('province');
		if ( isset($options_arr[$this->getCountry()]) ) {
			$options = $options_arr[$this->getCountry()];
		} else {
			$options = array();
		}

		if ( $this->Validator->inArrayKey(		$this->province_validator_label,
												$province,
												$this->province_validator_options_msg,
												$options ) ) {

			$this->data['province'] = $province;

			return TRUE;
		}

		return FALSE;
	}

	function getCountry() {
		if ( isset($this->data['country']) ) {
			return $this->data['country'];
		}

		return FALSE;
	}
	function setCountry($country) {
		$country = trim($country);

		$cf = new CompanyFactory();

		if ( $this->Validator->inArrayKey(		$this->country_validator_label,
												$country,
												$this->country_validator_options_msg,
												$cf->getOptions('country') ) ) {

			$this->data['country'] = $country;

			return TRUE;
		}

		return FALSE;
	}

	function getInvoiceDistrict() {
		if ( isset($this->data['invoice_district_id']) ) {
			return $this->data['invoice_district_id'];
		}

		return FALSE;
	}
	function setInvoiceDistrict($id) {
		$id = trim($id);

		Debug::Text('ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$idlf = new InvoiceDistrictListFactory();

		if ( $id == 0
				OR $this->Validator->isResultSetWithRows(	$this->invoicedistrict_validator_label,
													$idlf->getByID($id),
													$this->invoicedistrict_validator_msg
													) ) {

			$this->data['invoice_district_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getPostalCode() {
		if ( isset($this->data['postal_code']) ) {
			return $this->data['postal_code'];
		}

		return FALSE;
	}
	function setPostalCode($postal_code) {
		$postal_code = strtoupper( $this->Validator->stripSpaces($postal_code) );

		if 	(
				$postal_code == ''
				OR
				(
				$this->Validator->isPostalCode(		$this->postalcode_validator_label,
													$postal_code,
													$this->postalcode_validator_regex_msg,
													$this->getCountry() )
				AND
					$this->Validator->isLength(		$this->postalcode_validator_label,
													$postal_code,
													$this->postalcode_validator_length_msg,
													1,
													10)
				)
				) {

			$this->data['postal_code'] = $postal_code;

			return TRUE;
		}

		return FALSE;
	}

	function getWorkPhone() {
		if ( isset($this->data['work_phone']) ) {
			return $this->data['work_phone'];
		}

		return FALSE;
	}
	function setWorkPhone($work_phone) {
		$work_phone = trim($work_phone);

		if 	(	$work_phone == ''
				OR
				$this->Validator->isPhoneNumber(		$this->workphone_validator_label,
														$work_phone,
														$this->workphone_validator_isphonenumber_msg) ) {

			$this->data['work_phone'] = $work_phone;

			return TRUE;
		}

		return FALSE;
	}

	function getWorkPhoneExt() {
		if ( isset($this->data['work_phone_ext']) ) {
			return $this->data['work_phone_ext'];
		}

		return FALSE;
	}

	function setWorkPhoneExt($work_phone_ext) {
		$work_phone_ext = $this->Validator->stripNonNumeric( trim($work_phone_ext) );

		if ( 	$work_phone_ext == ''
				OR $this->Validator->isLength(		$this->workphoneext_validator_label,
													$work_phone_ext,
													$this->workphoneext_validator_length_msg,
													2,
													10) ) {

			$this->data['work_phone_ext'] = $work_phone_ext;

			return TRUE;
		}

		return FALSE;

	}

	function getMobilePhone() {
		return $this->data['mobile_phone'];
	}
	function setMobilePhone($mobile_phone) {
		$mobile_phone = trim($mobile_phone);

		if 	(	$mobile_phone == ''
					OR $this->Validator->isPhoneNumber(	$this->mobilephone_validator_label,
															$mobile_phone,
															$this->mobilephone_validator_isphonenumber_msg) ) {

			$this->data['mobile_phone'] = $mobile_phone;

			return TRUE;
		}

		return FALSE;
	}

	function getFaxPhone() {
		if ( isset($this->data['fax_phone']) ) {
			return $this->data['fax_phone'];
		}

		return FALSE;
	}
	function setFaxPhone($fax_phone) {
		$fax_phone = trim($fax_phone);

		if 	(	$fax_phone == ''
				OR
				$this->Validator->isPhoneNumber(		$this->faxphone_validator_label,
														$fax_phone,
														$this->faxphone_validator_isphonenumber_msg) ) {

			$this->data['fax_phone'] = $fax_phone;

			return TRUE;
		}

		return FALSE;
	}

	function getEmail() {
		if ( isset($this->data['email']) ) {
			return $this->data['email'];
		}

		return FALSE;
	}
	function setEmail($work_email) {
		$work_email = trim($work_email);

		if 	(	$work_email == ''
					OR	$this->Validator->isEmail(	$this->email_validator_label,
													$work_email,
													$this->email_validator_isemail_msg) ) {

			$this->data['email'] = $work_email;

			return TRUE;
		}

		return FALSE;
	}

	function isUniqueTypeDefault($client_id,$type_id) {
		$ph = array(
					'client_id' => $client_id,
					'type_id' => $type_id,
					);

		$query = 'select id from '. $this->getTable() .' where client_id = ? AND type_id = ? AND is_default = 1 AND deleted=0';
		$id = $this->db->GetOne($query, $ph);
		Debug::Arr($id,'Unique Client Contact Default Type: '. $type_id, __FILE__, __LINE__, __METHOD__,10);

		if ( $id === FALSE ) {
			return TRUE;
		} else {
			if ($id == $this->getId() ) {
				return TRUE;
			}
		}

		return FALSE;
	}

	function getDefault() {
		return $this->fromBool( $this->data['is_default'] );
	}
	function setDefault($bool) {

		if 	(
				$bool == TRUE
				AND
				$this->Validator->isTrue(		$this->default_validator_label,
												$this->isUniqueTypeDefault( $this->getClient(), $this->getType() ),
												$this->default_validator_unique_msg
												)
			) {

			$this->data['is_default'] = $this->toBool(TRUE);

			return TRUE;
		}

		$this->data['is_default'] = $this->toBool(FALSE);

		return TRUE;
	}

	function getOtherID1() {
		if ( isset($this->data['other_id1']) ) {
			return $this->data['other_id1'];
		}

		return FALSE;
	}
	function setOtherID1($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid1_validator_label,
											$value,
											$this->otherid1_validator_islength_msg,
											1,255) ) {

			$this->data['other_id1'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID2() {
		if ( isset($this->data['other_id2']) ) {
			return $this->data['other_id2'];
		}

		return FALSE;
	}
	function setOtherID2($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid2_validator_label,
											$value,
											$this->otherid2_validator_islength_msg,
											1,255) ) {

			$this->data['other_id2'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID3() {
		if ( isset($this->data['other_id3']) ) {
			return $this->data['other_id3'];
		}

		return FALSE;
	}
	function setOtherID3($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid3_validator_label,
											$value,
											$this->otherid3_validator_islength_msg,
											1,255) ) {

			$this->data['other_id3'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID4() {
		if ( isset($this->data['other_id4']) ) {
			return $this->data['other_id4'];
		}

		return FALSE;
	}
	function setOtherID4($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid4_validator_label,
											$value,
											$this->otherid4_validator_islength_msg,
											1,255) ) {

			$this->data['other_id4'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID5() {
		if ( isset($this->data['other_id5']) ) {
			return $this->data['other_id5'];
		}

		return FALSE;
	}
	function setOtherID5($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid5_validator_label,
											$value,
											$this->otherid5_validator_islength_msg,
											1,255) ) {

			$this->data['other_id5'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getNote() {
		if ( isset($this->data['note']) ) {
			return $this->data['note'];
		}
	}
	function setNote($val) {
		$val = trim($val);

		if 	(	$val == ''
				OR
				$this->Validator->isLength(		$this->note_validator_label,
												$val,
												$this->note_validator_length_msg,
												0,
												1024) ) {

			$this->data['note'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function getIncludeTaxPolicy() {
		$tpolf = new TaxPolicyObjectListFactory();
		$tpolf->getByObjectTypeIdAndObjectIdAndTypeId( 20, $this->getId(), 10 );
		foreach ($tpolf as $obj) {
			$list[] = $obj->getTaxPolicy();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setIncludeTaxPolicy($ids) {
		if ( $ids == FALSE ) {
			$ids = array(0);
		}

		Debug::text('Setting Tax Policy IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if (is_array($ids) and count($ids) > 0) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$tpolf = new TaxPolicyObjectListFactory();
				$tpolf->getByObjectTypeIdAndObjectIdAndTypeId( 20, $this->getId(), 10 );

				$tmp_ids = array();
				foreach ($tpolf as $obj) {
					$id = $obj->getTaxPolicy();
					Debug::text('Policy ID: '. $obj->getTaxPolicy() .' ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			}

			//Insert new mappings.
			//$ulf = new UserListFactory();
			$plf = new ProductListFactory();

			foreach ($ids as $id) {
				if ( isset($ids) AND !in_array($id, $tmp_ids) AND $id != 0) {
					$tpof = new TaxPolicyObjectFactory();
					$tpof->setObjectType( 20 );
					$tpof->setType( 10 );
					$tpof->setTaxPolicy( $id );
					$tpof->setObject( $this->getId() );

					//$obj = $plf->getById( $id );->getCurrent();
					$plf->getById( $id );
					if ( $plf->getRecordCount() > 0 ) {
						$name = $plf->getCurrent()->getName();
					} else {
						$name = 'N/A';
					}

					if ($this->Validator->isTrue(		$this->includetaxpolicy_validator_label,
														$tpof->Validator->isValid(),
														$this->includetaxpolicy_validator_msg.' ('. $name .')' )) {
						$tpof->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No User IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function getExcludeTaxPolicy() {
		$tpolf = new TaxPolicyObjectListFactory();
		$tpolf->getByObjectTypeIdAndObjectIdAndTypeId( 20, $this->getId(), 20 );
		foreach ($tpolf as $obj) {
			$list[] = $obj->getTaxPolicy();
		}

		if ( isset($list) ) {
			return $list;
		}

		return FALSE;
	}
	function setExcludeTaxPolicy($ids) {
		if ( $ids == FALSE ) {
			$ids = array(0);
		}

		Debug::text('Setting Tax Policy IDs : ', __FILE__, __LINE__, __METHOD__, 10);
		if (is_array($ids) and count($ids) > 0) {
			if ( !$this->isNew() ) {
				//If needed, delete mappings first.
				$tpolf = new TaxPolicyObjectListFactory();
				$tpolf->getByObjectTypeIdAndObjectIdAndTypeId( 20, $this->getId(), 20 );

				$tmp_ids = array();
				foreach ($tpolf as $obj) {
					$id = $obj->getTaxPolicy();
					Debug::text('Policy ID: '. $obj->getTaxPolicy() .' ID: '. $id, __FILE__, __LINE__, __METHOD__, 10);

					//Delete users that are not selected.
					if ( !in_array($id, $ids) ) {
						Debug::text('Deleting: '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$obj->Delete();
					} else {
						//Save ID's that need to be updated.
						Debug::text('NOT Deleting : '. $id, __FILE__, __LINE__, __METHOD__, 10);
						$tmp_ids[] = $id;
					}
				}
				unset($id, $obj);
			}

			//Insert new mappings.
			//$ulf = new UserListFactory();
			$plf = new ProductListFactory();

			foreach ($ids as $id) {
				if ( isset($ids) AND !in_array($id, $tmp_ids) AND $id != 0) {
					$tpof = new TaxPolicyObjectFactory();
					$tpof->setObjectType( 20 );
					$tpof->setType( 20 );
					$tpof->setTaxPolicy( $id );
					$tpof->setObject( $this->getId() );

					//$obj = $plf->getById( $id );->getCurrent();
					$plf->getById( $id );
					if ( $plf->getRecordCount() > 0 ) {
						$name = $plf->getCurrent()->getName();
					} else {
						$name = 'N/A';
					}

					if ($this->Validator->isTrue(		$this->excludetaxpolicy_validator_label,
														$tpof->Validator->isValid(),
														$this->excludetaxpolicy_validator_msg.' ('. $name .')' )) {
						$tpof->save();
					}
				}
			}

			return TRUE;
		}

		Debug::text('No User IDs to set.', __FILE__, __LINE__, __METHOD__, 10);
		return FALSE;
	}

	function postSave() {
		$this->removeCache( $this->getId() );

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Client Contact Information' , NULL, $this->getTable() );
	}

}
?>
