<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: RecurringScheduleTemplateFactory.class.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
class AccrualPolicyMilestoneFactory extends Factory {
	protected $table = 'accrual_policy_milestone';
	protected $pk_sequence_name = 'accrual_policy_milestone_id_seq'; //PK Sequence name

	protected $accrual_policy_obj = NULL;

	protected $length_of_service_unit_options = array(
										10 => 'Day(s)',
										20 => 'Week(s)',
										30 => 'Month(s)',
										40 => 'Year(s)',
									);

	protected $length_of_service_multiplier = array(
										10 => 1,
										20 => 7,
										30 => 30.4167,
										40 => 365.25,
									);

	protected $accrualpolicy_validator_msg = 'Accrual Policy is invalid';
	protected $accrualpolicy_validator_label = 'accrual_policy';

	protected $lengthofservice_validator_isnumeric_msg = 'Length of service is invalid';
	protected $lengthofservice_validator_label = 'length_of_service';

	protected $lengthofserviceunit_validator_options_msg = 'Incorrect Length of service unit';
	protected $lengthofserviceunit_validator_label = 'length_of_service_unit_id';

	protected $accrualrate_validator_isnumeric_msg = 'Incorrect Accrual Rate';
	protected $accrualrate_validator_label = 'accrual_rate';

	protected $minimumtime_validator_isnumeric_msg = 'Incorrect Minimum Time';
	protected $minimumtime_validator_label = 'minimum_time';

	protected $maximumtime_validator_isnumeric_msg = 'Incorrect Maximum Time';
	protected $maximumtime_validator_label = 'maximum_time';

	function getAccrualPolicyObject() {
		if ( is_object($this->accrual_policy_obj) ) {
			return $this->accrual_policy_obj;
		} else {
			$aplf = new AccrualPolicyListFactory();
			$aplf->getById( $this->getAccrualPolicyID() );
			if ( $aplf->getRecordCount() > 0 ) {
				$this->accrual_policy_obj = $aplf->getCurrent();
				return $this->accrual_policy_obj;
			}

			return FALSE;
		}
	}

	function getAccrualPolicy() {
		if ( isset($this->data['accrual_policy_id']) ) {
			return $this->data['accrual_policy_id'];
		}

		return FALSE;
	}
	function setAccrualPolicy($id) {
		$id = trim($id);

		$aplf = new AccrualPolicyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->accrualpolicy_validator_label,
													$aplf->getByID($id),
													$this->accrualpolicy_validator_msg
													) ) {

			$this->data['accrual_policy_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getLengthOfServiceDays() {
		if ( isset($this->data['length_of_service_days']) ) {
			return (int)$this->data['length_of_service_days'];
		}

		return FALSE;
	}
	function setLengthOfServiceDays($int) {
		$int = trim($int);

		if 	(	$int > 0
				AND
				$this->Validator->isFloat(			$this->lengthofservice_validator_label.$this->getLabelID(),
													$int,
													$this->lengthofservice_validator_isnumeric_msg) ) {
			Debug::text('Length of Service Days', __FILE__, __LINE__, __METHOD__, 10);
	
			$this->data['length_of_service_days'] = bcmul( $int, $this->length_of_service_multiplier[$this->getLengthOfServiceUnit()], 2);

			return TRUE;
		}

		return FALSE;
	}

	function getLengthOfService() {
		if ( isset($this->data['length_of_service']) ) {
			return (int)$this->data['length_of_service'];
		}

		return FALSE;
	}
	function setLengthOfService($int) {
		$int = trim($int);

		if 	(	$int > 0
				AND
				$this->Validator->isFloat(		$this->lengthofservice_validator_label.$this->getLabelID(),
													$int,
													$this->lengthofservice_validator_isnumeric_msg) ) {
			$this->data['length_of_service'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getLengthOfServiceUnit() {
		if ( isset($this->data['length_of_service_unit_id']) ) {
			return $this->data['length_of_service_unit_id'];
		}

		return FALSE;
	}
	function setLengthOfServiceUnit($value) {
		$value = trim($value);

		$key = Option::getByValue($value, $this->getOptions('length_of_service_unit') );
		if ($key !== FALSE) {
			$value = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->lengthofserviceunit_validator_label.$this->getLabelID(),
											$value,
											$this->lengthofserviceunit_validator_options_msg,
											$this->getOptions('length_of_service_unit')) ) {

			$this->data['length_of_service_unit_id'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getAccrualRate() {
		if ( isset($this->data['accrual_rate']) ) {
			return (int)$this->data['accrual_rate'];
		}

		return FALSE;
	}
	function setAccrualRate($int) {
		$int = trim($int);

		if 	(	$int > 0
				AND
				$this->Validator->isNumeric(		$this->accrualrate_validator_label.$this->getLabelID(),
													$int,
													$this->accrualrate_validator_isnumeric_msg) ) {
			$this->data['accrual_rate'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getMaximumTime() {
		if ( isset($this->data['maximum_time']) ) {
			return (int)$this->data['maximum_time'];
		}

		return FALSE;
	}
	function setMaximumTime($int) {
		$int = trim($int);

		if 	(	$this->Validator->isNumeric(		$this->maximumtime_validator_label.$this->getLabelID(),
													$int,
													$this->maximumtime_validator_isnumeric_msg) ) {
			$this->data['maximum_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getMinimumTime() {
		if ( isset($this->data['minimum_time']) ) {
			return (int)$this->data['minimum_time'];
		}

		return FALSE;
	}
	function setMinimumTime($int) {
		$int = trim($int);

		if 	(	$this->Validator->isNumeric(		$this->minimumtime_validator_label.$this->getLabelID(),
													$int,
													$this->minimumtime_validator_isnumeric_msg) ) {
			$this->data['minimum_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}
	
	function preSave() {
		//Set Length of service in days.
		$this->setLengthOfServiceDays( $this->getLengthOfService() );
		
		return TRUE;
	}
}
?>
