<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 589 $
 * $Id: ScheduleFactory.class.php 589 2007-01-30 22:44:08Z ipso $
 * $Date: 2007-01-30 14:44:08 -0800 (Tue, 30 Jan 2007) $
 */
class ScheduleFactory extends Factory {
	protected $table = 'schedule';
	protected $pk_sequence_name = 'schedule_id_seq'; //PK Sequence name

	protected $user_date_obj = NULL;
	protected $schedule_policy_obj = NULL;

	protected $status_options = array(
										10 => 'Working',
										20 => 'Absence'
									);

	protected $userdate_validator_msg = 'Invalid User/Date. Pay Period may be locked';
	protected $userdate_validator_label = 'user_date';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $starttime_validator_isdate_msg = 'Incorrect start time';
	protected $starttime_validator_isvalid_msg = 'Conflicting start time';
	protected $starttime_validator_label = 'start_time';

	protected $endtime_validator_isdate_msg = 'Incorrect end time';
	protected $endtime_validator_label = 'end_time';

	protected $totaltime_validator_isnumeric_msg = 'Incorrect total time';
	protected $totaltime_validator_label = 'total_time';

	protected $schedulepolicy_validator_msg = 'Schedule Policy is invalid';
	protected $schedulepolicy_validator_label = 'schedule_policy';

	protected $absencepolicy_validator_msg = 'Invalid Absence Policy ID';
	protected $absencepolicy_validator_label = 'absence_policy';

	protected $branch_validator_msg = 'Branch does not exist';
	protected $branch_validator_label = 'branch';

	protected $department_validator_msg = 'Department does not exist';
	protected $department_validator_label = 'department';

	protected $job_validator_msg = 'Job does not exist';
	protected $job_validator_label = 'job';

	protected $jobitem_validator_msg = 'Job Item does not exist';
	protected $jobitem_validator_label = 'job_item';

	function getSchedulePolicyObject() {
		if ( is_object($this->schedule_policy_obj) ) {
			return $this->schedule_policy_obj;
		} else {
			$splf = new SchedulePolicyListFactory();
			$splf->getById( $this->getSchedulePolicyID() );
			if ( $splf->getRecordCount() > 0 ) {
				$this->schedule_policy_obj = $splf->getCurrent();
				return $this->schedule_policy_obj;
			}

			return FALSE;
		}
	}


	function getUserDateObject() {
		if ( is_object($this->user_date_obj) ) {
			return $this->user_date_obj;
		} else {
			$udlf = new UserDateListFactory();
			$this->user_date_obj = $udlf->getById( $this->getUserDateID() )->getCurrent();

			return $this->user_date_obj;
		}
	}

	function findUserDate($user_id, $epoch) {
		//Get pay period start/continuous time
		$ppslf = new PayPeriodScheduleListFactory();
		$ppslf->getByUserId( $user_id );
		if ( $ppslf->getRecordCount() == 1 ) {
			$pps_obj = $ppslf->getCurrent();
			Debug::Text(' Pay Period Schedule Continuous Time: '. $pps_obj->getContinuousTime(), __FILE__, __LINE__, __METHOD__,10);

			$plf = new PunchListFactory();
			$plf->getFirstPunchByUserIDAndEpoch( $user_id, $epoch, $pps_obj->getContinuousTime() );

			if ( $plf->getRecordCount() > 0 ) {
				$p_obj = $plf->getCurrent();
				if ( ( $epoch - $p_obj->getTimeStamp() ) <= $pps_obj->getContinuousTime() ) {
					$user_date_id = $p_obj->getPunchControlObject()->getUserDateID();
					Debug::text(' User Date ID found: '. $user_date_id, __FILE__, __LINE__, __METHOD__,10);

					$this->setUserDateID( $user_date_id, TRUE );
					return TRUE;
				}
			}

		}

		return $this->setUserDate( $user_id, $epoch );
	}

	function setUserDate($user_id, $date) {
		$user_date_id = UserDateFactory::findOrInsertUserDate( $user_id, $date);
		Debug::text(' User Date ID: '. $user_date_id, __FILE__, __LINE__, __METHOD__,10);
		if ( $user_date_id != '' ) {
			$this->setUserDateID( $user_date_id );
			return TRUE;
		}
		Debug::text(' No User Date ID found', __FILE__, __LINE__, __METHOD__,10);

		return FALSE;
	}

	function getUserDateID() {
		if ( isset($this->data['user_date_id']) ) {
			return $this->data['user_date_id'];
		}

		return FALSE;
	}
	function setUserDateID($id, $skip_check = FALSE ) {
		$id = trim($id);

		$udlf = new UserDateListFactory();

		if (  $skip_check == TRUE
				OR
				$this->Validator->isResultSetWithRows(	$this->userdate_validator_label,
														$udlf->getByID($id),
														$this->userdate_validator_msg
														) ) {
			$this->data['user_date_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getStatus() {
		if ( isset($this->data['status_id']) ) {
			return $this->data['status_id'];
		}

		return FALSE;
	}
	function setStatus($status) {
		$status = trim($status);

		$key = Option::getByValue($status, $this->getOptions('status') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$status,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->data['status_id'] = $status;

			return FALSE;
		}

		return FALSE;
	}

	function getStartTime( $raw = FALSE ) {
		if ( isset($this->data['start_time']) ) {
			return TTDate::strtotime( $this->data['start_time'] );
			/*
			if ( $raw === TRUE) {
				return $this->data['start_time'];
			} else {
				return TTDate::strtotime( $this->data['start_time'] );
			}
			*/
		}

		return FALSE;
	}
	function setStartTime($epoch) {
		$epoch = trim($epoch);

		if 	(	$this->Validator->isDate(		$this->starttime_validator_label,
												$epoch,
												$this->starttime_validator_isdate_msg)

			) {

			$this->data['start_time'] = $epoch;

			return TRUE;
		}

		return FALSE;
	}

	function getEndTime( $raw = FALSE ) {
		if ( isset($this->data['end_time']) ) {
			return TTDate::strtotime( $this->data['end_time'] );
			/*
			if ( $raw === TRUE) {
				return $this->data['end_time'];
			} else {
				return TTDate::strtotime( $this->data['end_time'] );
			}
			*/
		}

		return FALSE;
	}
	function setEndTime($epoch) {
		$epoch = trim($epoch);

		if 	(	$this->Validator->isDate(		$this->endtime_validator_label,
												$epoch,
												$this->endtime_validator_isdate_msg)

			) {

			$this->data['end_time'] = $epoch;

			return TRUE;
		}

		return FALSE;
	}


	function calcTotalTime() {
		if ( $this->getSchedulePolicyObject() != FALSE ) {
			if ( $this->getSchedulePolicyObject()->getMealPolicyObject() != FALSE ) {
				$total_time = ( $this->getEndTime() - $this->getStartTime() ) - $this->getSchedulePolicyObject()->getMealPolicyObject()->getAmount();

				return $total_time;
			}
		}

		$total_time = ( $this->getEndTime() - $this->getStartTime() );

		return $total_time;
	}

	function getTotalTime() {
		if ( isset($this->data['total_time']) ) {
			return (int)$this->data['total_time'];
		}
		return FALSE;
	}
	function setTotalTime($int) {
		$int = (int)$int;

		if 	(	$this->Validator->isNumeric(		$this->totaltime_validator_label,
													$int,
													$this->totaltime_validator_isnumeric_msg) ) {
			$this->data['total_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}


	function getSchedulePolicyID() {
		if ( isset($this->data['schedule_policy_id']) ) {
			return $this->data['schedule_policy_id'];
		}

		return FALSE;
	}
	function setSchedulePolicyID($id) {
		$id = trim($id);

		if ( $id == '' OR empty($id) ) {
			$id = NULL;
		}

		$splf = new SchedulePolicyListFactory();

		if ( $id == NULL
				OR
				$this->Validator->isResultSetWithRows(	$this->schedulepolicy_validator_label,
														$splf->getByID($id),
														$this->schedulepolicy_validator_msg
													) ) {

			$this->data['schedule_policy_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getAbsencePolicyID() {
		if ( isset($this->data['absence_policy_id']) ) {
			return $this->data['absence_policy_id'];
		}

		return FALSE;
	}
	function setAbsencePolicyID($id) {
		$id = trim($id);

		if ( $id == '' OR empty($id) ) {
			$id = NULL;
		}

		$aplf = new AbsencePolicyListFactory();

		if (	$id == NULL
				OR
				$this->Validator->isResultSetWithRows(	$this->absencepolicy_validator_label,
														$aplf->getByID($id),
														$this->absencepolicy_validator_msg
														) ) {
			$this->data['absence_policy_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getBranch() {
		if ( isset($this->data['branch_id']) ) {
			return $this->data['branch_id'];
		}

		return FALSE;
	}
	function setBranch($id) {
		$id = trim($id);

		$blf = new BranchListFactory();

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->branch_validator_label,
														$blf->getByID($id),
														$this->branch_validator_msg
														) ) {
			$this->data['branch_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getDepartment() {
		if ( isset($this->data['department_id']) ) {
			return $this->data['department_id'];
		}

		return FALSE;
	}
	function setDepartment($id) {
		$id = trim($id);

		$dlf = new DepartmentListFactory();

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->department_validator_label,
														$dlf->getByID($id),
														$this->department_validator_msg
														) ) {
			$this->data['department_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getJob() {
		if ( isset($this->data['job_id']) ) {
			return $this->data['job_id'];
		}

		return FALSE;
	}
	function setJob($id) {
		$id = trim($id);

		if ( $id == FALSE OR $id == 0 OR $id == '' ) {
			$id = 0;
		}
		
		if ( isProfessionalEditionAvailable() == TRUE ) {
			$jlf = new JobListFactory();
		}

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->job_validator_label,
														$jlf->getByID($id),
														$this->job_validator_msg
														) ) {
			$this->data['job_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getJobItem() {
		if ( isset($this->data['job_item_id']) ) {
			return $this->data['job_item_id'];
		}

		return FALSE;
	}
	function setJobItem($id) {
		$id = trim($id);

		if ( $id == FALSE OR $id == 0 OR $id == '' ) {
			$id = NULL;
		}

		if ( isProfessionalEditionAvailable() == TRUE ) {
			$jilf = new JobItemListFactory();
		}

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->jobitem_validator_label,
														$jilf->getByID($id),
														$this->jobitem_validator_msg
														) ) {
			$this->data['job_item_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function inSchedule( $epoch ) {
		if ( $epoch >= $this->getStartTime() AND $epoch <= $this->getEndTime() ) {
			Debug::text('aWithin Schedule: '. $epoch, __FILE__, __LINE__, __METHOD__,10);

			return TRUE;
		} elseif ( $this->inStartWindow( $epoch ) OR $this->inStopWindow( $epoch) )  {
			Debug::text('bWithin Schedule: '. $epoch, __FILE__, __LINE__, __METHOD__,10);

			return TRUE;
		}

		return FALSE;
	}

	function inStartWindow( $epoch ) {
		Debug::text(' Epoch: '. $epoch, __FILE__, __LINE__, __METHOD__,10);

		if ( $epoch == '' ) {
			return FALSE;
		}

		if (	$this->getSchedulePolicyObject() !== FALSE
				AND
				(
					$epoch >= ( $this->getStartTime() - $this->getSchedulePolicyObject()->getStartStopWindow() )
					AND
					$epoch <= ( $this->getStartTime() + $this->getSchedulePolicyObject()->getStartStopWindow() )
				)
			) {
			Debug::text(' Within Start/Stop window: '. $this->getSchedulePolicyObject()->getStartStopWindow() , __FILE__, __LINE__, __METHOD__,10);

			return TRUE;
		} else {
			Debug::text(' NOT Within Start/Stop window.', __FILE__, __LINE__, __METHOD__,10);
			return FALSE;
		}

		return FALSE;
	}

	function inStopWindow( $epoch ) {
		Debug::text(' Epoch: '. $epoch, __FILE__, __LINE__, __METHOD__,10);

		if ( $epoch == '' ) {
			return FALSE;
		}

		if (	$this->getSchedulePolicyObject() !== FALSE
				AND
				(
					$epoch >= ( $this->getEndTime() - $this->getSchedulePolicyObject()->getStartStopWindow() )
					AND
					$epoch <= ( $this->getEndTime() + $this->getSchedulePolicyObject()->getStartStopWindow() )
				)

			) {
			Debug::text(' Within Start/Stop window: '. $this->getSchedulePolicyObject()->getStartStopWindow() , __FILE__, __LINE__, __METHOD__,10);

			return TRUE;
		} else {
			Debug::text(' NOT Within Start/Stop window.', __FILE__, __LINE__, __METHOD__,10);
			return FALSE;
		}

		return FALSE;
	}

	function mergeScheduleArray($schedule_shifts, $recurring_schedule_shifts) {
		//Debug::text('Merging Schedule, and Recurring Schedule Shifts: ', __FILE__, __LINE__, __METHOD__, 10);

		$ret_arr = $schedule_shifts;

		//Debug::Arr($schedule_shifts, '(c) Schedule Shifts: ', __FILE__, __LINE__, __METHOD__, 10);

		if ( is_array($recurring_schedule_shifts) AND count($recurring_schedule_shifts) > 0 ) {
			foreach( $recurring_schedule_shifts as $date_stamp => $day_shifts_arr ) {
				//Debug::text('----------------------------------', __FILE__, __LINE__, __METHOD__, 10);
				//Debug::text('Date Stamp: '. TTDate::getDate('DATE+TIME', $date_stamp). ' Epoch: '. $date_stamp , __FILE__, __LINE__, __METHOD__, 10);
				//Debug::Arr($schedule_shifts[$date_stamp], 'Date Arr: ', __FILE__, __LINE__, __METHOD__, 10);
				foreach( $day_shifts_arr as $key => $shift_arr ) {

					if ( isset($ret_arr[$date_stamp]) ) {
						//Debug::text('Already Schedule Shift on this day: '. TTDate::getDate('DATE', $date_stamp) , __FILE__, __LINE__, __METHOD__, 10);

						//Loop through each shift on this day, and check for overlaps
						//Only include the recurring shift if ALL times DO NOT overlap
						$overlap = 0;
						foreach( $ret_arr[$date_stamp] as $tmp_shift_arr ) {
							if ( TTDate::isTimeOverLap( $shift_arr['start_time'], $shift_arr['end_time'], $tmp_shift_arr['start_time'], $tmp_shift_arr['end_time']) ) {
								//Debug::text('Times OverLap: '. TTDate::getDate('DATE+TIME', $shift_arr['start_time']) , __FILE__, __LINE__, __METHOD__, 10);
								$overlap++;
							} else {
								//Debug::text('Times DO NOT OverLap: '. TTDate::getDate('DATE+TIME', $shift_arr['start_time']) , __FILE__, __LINE__, __METHOD__, 10);

							}
						}

						if ( $overlap == 0 ) {
							Debug::text('NO Times OverLap, using recurring schedule: '. TTDate::getDate('DATE+TIME', $shift_arr['start_time']) , __FILE__, __LINE__, __METHOD__, 10);
							$ret_arr[$date_stamp][] = $shift_arr;
						}
					} else {
						//Debug::text('No Schedule Shift on this day: '. TTDate::getDate('DATE', $date_stamp) , __FILE__, __LINE__, __METHOD__, 10);
						$ret_arr[$date_stamp][] = $shift_arr;
					}
				}
			}
		}

		return $ret_arr;
	}

	function getScheduleArray( $user_ids, $start_date, $end_date ) {
		global $current_user;

		if ( $user_ids == '' OR ( is_array($user_ids) AND count($user_ids) == 0 ) ) {
			return FALSE;
		}

		if ( $start_date == '' ) {
			return FALSE;
		}

		if ( $end_date == '' ) {
			return FALSE;
		}

		//Convert to array for one user_id
		if ( !is_array($user_ids) ) {
			$user_ids = array($user_ids);
		}

		$slf = new ScheduleListFactory();
		$rsclf = new RecurringScheduleControlListFactory();
		$ulf = new UserListFactory();
		$hlf = new HolidayListFactory();

		$blf = new BranchListFactory();
		$branch_options = $blf->getByCompanyIdArray( $current_user->getCompany(), FALSE );

		$dlf = new DepartmentListFactory();
		$department_options = $dlf->getByCompanyIdArray( $current_user->getCompany(), FALSE );

		foreach( $user_ids as $user_id ) {
			Debug::text('User ID: '. $user_id , __FILE__, __LINE__, __METHOD__, 10);
			$ulf->getByIdAndCompanyId( $user_id, $current_user->getCompany() );
			if ( $ulf->getRecordCount() > 0 ) {
				$user_obj = $ulf->getCurrent();
			}
			//Grab all schedule rows in this time period.

			$slf->getByUserIdAndStartDateAndEndDate( $user_id, $start_date, $end_date );

			if ( $slf->getRecordCount() > 0 ) {
				foreach( $slf as $s_obj ) {
					//Debug::text('Schedule ID: '. $s_obj->getId() .' Row: '. $s_obj->getStartTime(), __FILE__, __LINE__, __METHOD__, 10);
					$schedule_shifts[TTDate::getBeginDayEpoch($s_obj->getStartTime())][] = array(
														'id' => $s_obj->getID(),
														'user_id' => $user_id,
														'user_full_name' => $user_obj->getFullName(),
														'status_id' => $s_obj->getStatus(),
														'start_time' => $s_obj->getStartTime(),
														'end_time' => $s_obj->getEndTime(),
														'total_time' => $s_obj->getTotalTime(),
														'schedule_policy_id' => $s_obj->getSchedulePolicyID(),
														'absence_policy_id' => $s_obj->getAbsencePolicyID(),
														'branch_id' => $s_obj->getBranch(),
														'branch' => Option::getByKey($s_obj->getBranch(), $branch_options, NULL ),
														'department_id' => $s_obj->getDepartment(),
														'department' =>  Option::getByKey($s_obj->getDepartment(), $department_options, NULL ),
													);

				}

				//Debug::Arr($schedule_shifts, 'Schedule Shifts: ', __FILE__, __LINE__, __METHOD__, 10);
			} else {
				$schedule_shifts = array();
			}

			//Grab Recurring Schedule data for this time period.
			$rsclf->getByUserIDAndStartDateAndEndDate( $user_id, $start_date, $end_date );
			if ( $rsclf->getRecordCount() > 0 ) {
				Debug::text('Found Recurring Schedule for this time period: ', __FILE__, __LINE__, __METHOD__, 10);

				//Get all holidays in this schedule date range.
				$hlf->getByPolicyGroupUserIdAndStartDateAndEndDate( $user_id, $start_date, $end_date );
				if ( $hlf->getRecordCount() > 0 ) {
					foreach( $hlf as $h_obj ) {
						if ( is_object( $h_obj->getHolidayPolicyObject() ) ) {						
							$holiday_data[$h_obj->getDateStamp()] = (int)$h_obj->getHolidayPolicyObject()->getDefaultScheduleStatus();
						} else {
							$holiday_data[$h_obj->getDateStamp()] = 10; //Working
						}
					}
				}

				$recurring_schedule_shifts = array();
				foreach( $rsclf as $rsc_obj ) {
					Debug::text('Recurring Schedule ID: '. $rsc_obj->getID() , __FILE__, __LINE__, __METHOD__, 10);
					$tmp_recurring_schedule_shifts = $rsc_obj->getShiftsByStartDateAndEndDate( $start_date, $end_date );

					if ( is_array($tmp_recurring_schedule_shifts) ) {
						foreach( $tmp_recurring_schedule_shifts as $day_epoch => $day_tmp_recurring_schedule_shift ) {							
							if ( is_array($day_tmp_recurring_schedule_shift) ) {
								foreach( $day_tmp_recurring_schedule_shift as $key => $dummy ) {
									$tmp_recurring_schedule_shifts[$day_epoch][$key]['user_id'] = $user_id;
									$tmp_recurring_schedule_shifts[$day_epoch][$key]['user_full_name'] = $user_obj->getFullName();

									if ( isset($holiday_data[$day_epoch]) ) {
										//We have to assume they are eligible, because we really won't know
										//if they will have worked enough days or not. We could assume they
										//work whatever their schedule is, but chances are they will be eligible then anyways.
										$tmp_recurring_schedule_shifts[$day_epoch][$key]['status_id'] = $holiday_data[$day_epoch];
									}
									
									//Default branch
									if ( $tmp_recurring_schedule_shifts[$day_epoch][$key]['branch_id'] == -1 ) {
										$tmp_recurring_schedule_shifts[$day_epoch][$key]['branch'] = Option::getByKey($user_obj->getDefaultBranch(), $branch_options, NULL );
									} else {
										$tmp_recurring_schedule_shifts[$day_epoch][$key]['branch'] = Option::getByKey($tmp_recurring_schedule_shifts[$day_epoch][$key]['branch_id'], $branch_options, NULL );
									}

									//Default department
									if ( $tmp_recurring_schedule_shifts[$day_epoch][$key]['department_id'] == -1 ) {
										$tmp_recurring_schedule_shifts[$day_epoch][$key]['department'] = Option::getByKey($user_obj->getDefaultDepartment(), $department_options, NULL );
									} else {
										//$tmp_recurring_schedule_shifts[$day_epoch][$key]['department'] = $department_options[$tmp_recurring_schedule_shifts[$day_epoch][$key]['department_id']];
										$tmp_recurring_schedule_shifts[$day_epoch][$key]['department'] = Option::getByKey($tmp_recurring_schedule_shifts[$day_epoch][$key]['department_id'], $department_options, NULL );
									}
								}
							}
							
							unset($holiday_obj);
						}
					}
					//var_dump($tmp_recurring_schedule_shifts);
					//Merge recurring schedules.
					$recurring_schedule_shifts = $this->mergeScheduleArray( $recurring_schedule_shifts, $tmp_recurring_schedule_shifts);
				}
			} else {
				Debug::text('DID NOT find Recurring Schedule for this time period: ', __FILE__, __LINE__, __METHOD__, 10);
				$recurring_schedule_shifts = array();
			}

			//Debug::Arr($recurring_schedule_shifts, 'Recurring Schedule Shifts: ', __FILE__, __LINE__, __METHOD__, 10);
		}

		//Debug::Arr($schedule_shifts, '(b) Schedule Shifts: ', __FILE__, __LINE__, __METHOD__, 10);

		//Merge Schedule Shifts and Recurring Schedule Shifts.
		$shifts = $this->mergeScheduleArray( $schedule_shifts, $recurring_schedule_shifts);

		//Debug::Arr($shifts, 'All Shifts: ', __FILE__, __LINE__, __METHOD__, 10);

		if ( isset($shifts) ) {
			return $shifts;
		}

		return FALSE;
	}

	function getEnableReCalculateDay() {
		if ( isset($this->recalc_day) ) {
			return $this->recalc_day;
		}

		return FALSE;
	}
	function setEnableReCalculateDay($bool) {
		$this->recalc_day = $bool;

		return TRUE;
	}

	function handleDayBoundary() {
		Debug::Arr($this->getStartTime(), 'Start Time: '. TTDate::getDate('DATE+TIME', $this->getStartTime()), __FILE__, __LINE__, __METHOD__, 10);
		Debug::Arr($this->getEndTime(), 'End Time: '. TTDate::getDate('DATE+TIME', $this->getEndTime()), __FILE__, __LINE__, __METHOD__, 10);

		//This used to be done in Validate, but needs to be done in preSave too.
		if ( $this->getEndTime() < $this->getStartTime() ) {
			Debug::Text('EndTime spans midnight boundary! Increase by 24hrs ', __FILE__, __LINE__, __METHOD__,10);
			$this->setEndTime( $this->getEndTime() + 86400 ); //End time spans midnight, add 24hrs.
		}

		return TRUE;
	}

	function Validate() {
		Debug::Text('User Date ID: '. $this->getUserDateID(), __FILE__, __LINE__, __METHOD__,10);

		$this->handleDayBoundary();

		//Make sure we're not conflicting with any other schedule shifts.
		$slf = new ScheduleListFactory();
		$conflicting_schedule_shift_obj = $slf->getConflictingByUserDateIdAndStartDateAndEndDate( $this->getUserDateID(), $this->getStartTime(), $this->getEndTime() );

		if ( is_object($conflicting_schedule_shift_obj) ) {
			$conflicting_schedule_shift_obj = $conflicting_schedule_shift_obj->getCurrent();

			if ( $conflicting_schedule_shift_obj->isNew() === FALSE
					AND $conflicting_schedule_shift_obj->getId() != $this->getId() ) {
				Debug::text('Conflicting Schedule Shift ID:'. $conflicting_schedule_shift_obj->getId() .' Schedule Shift ID: '. $this->getId() , __FILE__, __LINE__, __METHOD__, 10);
				$this->Validator->isTrue(		$this->starttime_validator_label,
												FALSE,
												$this->starttime_validator_isvalid_msg);
			}
		}

		if ( $this->getUserDateID() == FALSE ) {
			Debug::Text('UserDateID is INVALID! ', __FILE__, __LINE__, __METHOD__,10);
			$this->Validator->isTrue(		$this->userdate_validator_label,
											FALSE,
											$this->userdate_validator_msg);
		}

		return TRUE;
	}

	function preSave() {
		$this->handleDayBoundary();

		if ( $this->getTotalTime() == FALSE ) {
			$this->setTotalTime( $this->calcTotalTime() );
		}

		if ( $this->getStatus() == 10 ) {
			$this->setAbsencePolicyID( NULL );
		}
		return TRUE;
	}

	function postSave() {
		Debug::text(' postSave()', __FILE__, __LINE__, __METHOD__,10);

		if ( $this->getEnableReCalculateDay() == TRUE ) {
			//Calculate total time. Mainly for docked.
			UserDateTotalFactory::reCalculateDay( $this->getUserDateID(), TRUE, FALSE );
			//ExceptionPolicyFactory::calcExceptions( $this->getUserDateID() );
		}

		return TRUE;
	}
}
?>
