<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 572 $
 * $Id: EditPermission.php 572 2007-01-29 16:50:02Z ipso $
 * $Date: 2007-01-29 08:50:02 -0800 (Mon, 29 Jan 2007) $
 */
require_once('../../includes/global.inc.php');
require_once(Environment::getBasePath() .'includes/Interface.inc.php');

//Debug::setVerbosity(11);

if ( !$permission->Check('permission','enabled')
		OR !( $permission->Check('permission','edit') OR $permission->Check('permission','edit_own') ) ) {

	$permission->Redirect( FALSE ); //Redirect
}

$title = 'Edit Employee Permissions';
$smarty->assign('title', $title);

/*
 * Get FORM variables
 */
extract	(FormVariables::GetVariables(
										array	(
												'action',
												'id',
												'group_id',
												'data',
												'old_data',
												) ) );


switch (strtolower($action)) {
	case 'apply':
		//Debug::setVerbosity(11);
		Debug::Text('Apply', __FILE__, __LINE__, __METHOD__,10);

		if ( !isset($data['preset_flags']) ) {
			$data['preset_flags'] = array();
		}

		if ( DEMO_MODE == FALSE AND isset($data['user_ids']) AND count($data['user_ids']) > 0
			AND isset($data['preset']) ) {

			$pf = new PermissionFactory();
			$pf->applyPreset($data['user_ids'], $data['preset'], $data['preset_flags']);
		}

		Redirect::Page( URLBuilder::getURL( array('id'=> $id), 'EditPermission.php') );

		break;
	case 'submit':
		//Debug::setVerbosity(11);

		Debug::Text('Submit!', __FILE__, __LINE__, __METHOD__,10);

		if ( DEMO_MODE == FALSE AND isset($data['user_ids']) AND count($data['user_ids']) > 0
				AND isset($data['permissions']) AND count($data['permissions']) > 0) {
			$pf = new PermissionFactory();

			$pf->StartTransaction();

			foreach ($data['user_ids'] as $user_id) {
				Debug::Text('User ID: '. $user_id, __FILE__, __LINE__, __METHOD__,10);

				if ( $user_id == -1 ) {
					$company_id = $current_company->getId();
				} else {
					$ulf = new UserListFactory();
					if ( $permission->Check('company','edit') ) {
						$company_id = $ulf->getById ( $user_id )->getCurrent()->getCompany();
					} else {
						$company_id = $current_company->getId();
					}
				}

				$clf = new CompanyListFactory();
				$clf->getById( $company_id );
				if ( $clf->getRecordCount() > 0 ) {
					$company_obj = $clf->getCurrent();

					//Don't Delete all previous permissions, do that in the Permission class.
					foreach ($data['permissions'] as $section => $permissions) {
						Debug::Text('  Section: '. $section, __FILE__, __LINE__, __METHOD__,10);

						foreach ($permissions as $name => $value) {
							Debug::Text('     Name: '. $name .' - Value: '. $value, __FILE__, __LINE__, __METHOD__,10);
							if ( isset($old_data['permissions'][$section][$name])
									AND $value != $old_data['permissions'][$section][$name]
									AND $pf->isIgnore( $section, $name, $company_obj->getProductEdition() ) == FALSE
									) {

								if ( $value == 0 OR $value == 1 ) {
									Debug::Text('    Modifying/Adding Permission: '. $name .' - Value: '. $value, __FILE__, __LINE__, __METHOD__,10);
									$pf->setCompany( $company_id );
									$pf->setUser( $user_id );
									$pf->setSection( $section );
									$pf->setName( $name );
									$pf->setValue( $value );

									if ( $pf->isValid() ) {
										$pf->save();
									}

								} else {
									Debug::Text('    Deleting Permission: '. $name .' - Value: '. $value, __FILE__, __LINE__, __METHOD__,10);

									//Delete the specific permission if it exists.
									$plf = new PermissionListFactory();
									$obj = $plf->getByCompanyIdAndUserIdAndSectionAndName( $company_id, $user_id, $section, $name )->getCurrent();
									//var_dump($obj);
									$obj->delete();
									//$obj->removeCache( $obj->getCacheID() );
								}
							} else {
								Debug::Text('     Permission didnt change... Skipping', __FILE__, __LINE__, __METHOD__,10);
							}
						}
					}

					Debug::Text('    Clear Cache Here!', __FILE__, __LINE__, __METHOD__,10);
					$pf->clearCache( $user_id, $current_company->getID() );

				}

			}

			$pf->CommitTransaction();
		}

		Redirect::Page( URLBuilder::getURL( array('id'=> $id), 'EditPermission.php') );

		break;

	default:
		BreadCrumb::setCrumb($title);

		$pf = new PermissionFactory();
		$plf = new PermissionListFactory();
		$ulf = new UserListFactory();

		if ( $permission->Check('company','edit') == TRUE ) {
			$company_id = $ulf->getById ( $id )->getCurrent()->getCompany();
		}

		if ( !isset($company_id) OR $company_id == '' ) {
			$company_id = $current_company->getId();
		}

		//Get company object
		$clf = new CompanyListFactory();
		$clf->getById( $company_id );
		if ( $clf->getRecordCount() > 0 ) {
			$company_obj = $clf->getCurrent();
		}

		$plf->getAllPermissionsByCompanyIdAndUserId($company_id, $id);
		if ( $plf->getRecordCount() > 0 ) {
			foreach($plf as $p_obj) {
				foreach($plf as $p_obj) {
					$current_permissions[$p_obj->getSection()][$p_obj->getName()] = $p_obj;
				}
			}
		}
		//var_dump($current_permissions);

		$section_groups = $pf->getOptions('section_group');
		$section_group_map = $pf->getOptions('section_group_map');
		$sections = $pf->getOptions('section');
		$names = $pf->getOptions('name');

		//Trim out ignored sections
		foreach( $section_groups as $section_group_key => $section_group_value ) {
			if ( $pf->isIgnore( $section_group_key, NULL, $company_obj->getProductEdition() ) == TRUE ) {
				unset($section_groups[$section_group_key]);
			}
		}
		unset($section_group_key, $section_group_value);

		if ( !isset($group_id) OR !isset($section_groups[$group_id]) ) {
			$group_id = 0; //None
		}
		Debug::Text('Group ID: '. $group_id, __FILE__, __LINE__, __METHOD__,10);

		foreach ($names as $section => $permission_arr) {
			if (
					( $pf->isIgnore( $section, NULL, $company_obj->getProductEdition() ) == FALSE )
					AND
					( ( $group_id == 'all' AND $group_id !== 0 ) OR ( isset($section_group_map[$group_id]) AND in_array($section,$section_group_map[$group_id]) ) )
					) {

				foreach($permission_arr as $name => $display_name) {

						if ( isset($current_permissions[$section][$name]) ) {
							$permission_result_obj = $current_permissions[$section][$name];
							//$permission_result_obj = $permission->Query($section, $name, $id, $company_id);

							Debug::Text(' Permission Check Section: '. $section .' - Name: '. $name .' - Get User: '. $permission_result_obj->getUser(), __FILE__, __LINE__, __METHOD__,10);

							if ( $id != -1 AND ( $permission_result_obj->getUser() == -1 OR $permission_result_obj->getUser() == FALSE ) ) {
								Debug::Text(' Permission Check -1: ', __FILE__, __LINE__, __METHOD__,10);
								$default_result = TRUE;

							} else {
								Debug::Text(' Permission Check Value: '. $permission_result_obj->getValue(), __FILE__, __LINE__, __METHOD__,10);
								$default_result = FALSE;
							}
							$permission_result = $permission_result_obj->getValue();

							$permissions[] = array('name' => $name, 'display_name' => $display_name, 'default' => $default_result, 'result' => $permission_result);
						} elseif ( $pf->isIgnore( $section, $name, $company_obj->getProductEdition() ) == FALSE ) {
							$permissions[] = array('name' => $name, 'display_name' => $display_name, 'default' => TRUE, 'result' => NULL );
						}
				}

				//If you get a index error just below here, you forgot to
				//enter the section name in PayStubFactory.class.php
				$permission_data[] = array(
											'name' => $section,
											'display_name' => $sections[$section],
											'permissions' => $permissions
											);

				unset($permissions);
			}

		}

		//var_dump($permission_data);
		$preset_options = $pf->getOptions('preset');
		$smarty->assign_by_ref('preset_options', $preset_options );

		$smarty->assign_by_ref('section_group_options', $section_groups );


		$user_options = UserListFactory::getByCompanyIdArray( $company_id, FALSE);
		$user_options = Misc::prependArray( array( -1 => '-- Default --'), $user_options );
		//$user_options[-1] = '- Default -'; //Allow them to change the default user perms

		$smarty->assign_by_ref('user_options', $user_options);

		$smarty->assign_by_ref('permission_data', $permission_data);

		$smarty->assign_by_ref('ignore_permissions', $ignore_permissions);
		$smarty->assign_by_ref('id', $id);
		$smarty->assign_by_ref('group_id', $group_id);
		$smarty->assign_by_ref('product_edition', $company_obj->getProductEdition() );

		break;
}

$smarty->assign_by_ref('pf', $pf);

$smarty->display('permission/EditPermission.tpl');
?>